import { Request, Response } from "express";
import prisma, { Prisma } from "@server/prisma";
import { datatableRequest } from "@server/controllers/guild/api/dt.module";
import { getLogger } from "@server/../log";

const logger = getLogger(__filename);

// TODO: this doesn't account for guild ID or permissions
export const get = async (request: Request, response: Response) => {
    logger.info(`Getting filter: ${request.query.id}`);

    if (!request.query.id) {
        response.status(400).json({ error: "missing 'id' query" });
        return;
    }

    const filter = await prisma.filter.findUnique({
        where: { id: Number(request.query.id) }
    });

    if (!filter) {
        response.status(404).json({ message: "no result found" });
        return;
    }

    response.json(filter);
};

export const post = async (request: Request, response: Response) => {
    logger.info(`Posting filter: ${request.body.value} - ${request.params.guildId}`);

    const body = {
        ...request.body,
        is_insensitive: request.body.is_insensitive === "on",
        is_whitelist: request.body.is_whitelist === "on"
    };

    const createInputData: Prisma.FilterUncheckedCreateInput = {
        guild_id: request.params.guildId,
        name: body.name,
        value: body.value,
        matching_algorithm: body.matching_algorithm,
        is_insensitive: body.is_insensitive,
        is_whitelist: body.is_whitelist
    };

    try {
        const createResponse = await prisma.filter.create({ data: createInputData });
        response.status(201).json(createResponse);
    } catch (error) {
        logger.error(error);
        const isPrismaError = error instanceof Prisma.PrismaClientKnownRequestError;
        response.status(500).json({ error: isPrismaError ? error.message : error });
    }
};

export const patch = async (request: Request, response: Response) => {
    logger.info(`Patching filter: ${request.body.id} - ${request.params.guildId}`);

    const body = {
        ...request.body,
        is_insensitive: request.body.is_insensitive === "on",
        is_whitelist: request.body.is_whitelist === "on"
    };

    const updateInputData: Prisma.FilterUncheckedUpdateInput = {
        guild_id: request.params.guildId,
        name: body.name,
        value: body.value,
        matching_algorithm: body.matching_algorithm,
        is_insensitive: body.is_insensitive,
        is_whitelist: body.is_whitelist
    };

    try {
        const updateArgs = { where: { id: Number(body.id) }, data: updateInputData };
        const updateResponse = await prisma.filter.update(updateArgs);
        response.status(200).json(updateResponse);
    } catch (error) {
        logger.error(error);
        const isPrismaError = error instanceof Prisma.PrismaClientKnownRequestError;
        response.status(500).json({ error: isPrismaError ? error.message : error });
    }
}

export const del = async (request: Request, response: Response) => {
    logger.info(`Deleting filter(s): ${request.body.ids} - ${request.params.guildId}`);

    const ids = request.body.ids?.map((id: string) => Number(id));

    if (!ids) {
        response.status(400).json({ error: "Couldn't parse ID's from request body" });
        return;
    }

    try {
        const deleteArgs = { where: { guild_id: request.params.guildId, id: { in: ids } } };
        await prisma.filter.deleteMany(deleteArgs);
        response.status(204).send();
    } catch (error) {
        logger.error(error);
        const isPrismaError = error instanceof Prisma.PrismaClientKnownRequestError;
        response.status(500).json({ error: isPrismaError ? error.message : error });
    }
};

export const datatable = async (request: Request, response: Response) => {
    return await datatableRequest(
        request,
        response,
        prisma.filter,
        [{ updated_at: "desc" }, { id: "asc" }],
        {},
        { guild_id: request.params.guildId }
    );
};

export const select = async (request: Request, response: Response) => {
    const data = await prisma.filter.findMany({
        where: {
            guild_id: request.params.guildId,
            name: { contains: `${request.query.search}` }
        }
    });

    // Preline Bug: https://github.com/htmlstreamofficial/preline/issues/567
    // The returned data must have a "title" key, otherwise the advanced
    // select component with 'tags' mode will have no title, regardless of
    // mapping.
    const modifiedResults = data.map(filter => ({
        ...filter,
        title: filter.name
    }));

    response.status(200).json(modifiedResults);
};

export default { get, post, patch, del, datatable, select };