import { Request, Response } from "express";
import prisma, { Prisma } from "@server/prisma";
import { datatableRequest } from "@server/controllers/guild/api/dt.module";
import { getLogger } from "@server/../log";

const logger = getLogger(__filename);

export const get = async (request: Request, response: Response) => {
    logger.info(`Getting style: ${request.query.id}`);

    if (!request.query.id) {
        response.status(400).json({ error: "missing 'id' query" });
        return;
    }

    const style = await prisma.messageStyle.findUnique({
        where: { id: Number(request.query.id) },
    });

    if (!style) {
        response.status(404).json({ message: "no result found" });
        return;
    }

    response.json(style);
};

export const post = async (request: Request, response: Response) => {
    logger.info(`Posting style: ${request.body.colour} - ${request.params.guildId}`);

    const body = {
        ...request.body,
        show_author: request.body.show_author === "on",
        show_image: request.body.show_image === "on",
        show_thumbnail: request.body.show_thumbnail === "on",
        show_footer: request.body.show_footer === "on",
        show_timestamp: request.body.show_timestamp === "on"
    };

    const createInputData: Prisma.MessageStyleUncheckedCreateInput = {
        guild_id: request.params.guildId,
        name: body.name,
        colour: body.colour,
        show_author: body.show_author,
        show_image: body.show_image,
        show_thumbnail: body.show_thumbnail,
        show_footer: body.show_footer,
        show_timestamp: body.show_timestamp,
        title_mutator: body.title_mutator || null,
        description_mutator: body.description_mutator || null
    };

    try {
        const createResponse = await prisma.messageStyle.create({ data: createInputData });
        response.status(201).json(createResponse);
    } catch (error) {
        logger.error(error);
        const isPrismaError = error instanceof Prisma.PrismaClientKnownRequestError;
        response.status(500).json({ error: isPrismaError ? error.message : error });
    }
};

export const patch = async (request: Request, response: Response) => {
    logger.info(`Patching style: ${request.body.id} - ${request.params.guildId}`);

    const body = {
        ...request.body,
        show_author: request.body.show_author === "on",
        show_image: request.body.show_image === "on",
        show_thumbnail: request.body.show_thumbnail === "on",
        show_footer: request.body.show_footer === "on",
        show_timestamp: request.body.show_timestamp === "on"
    };

    const updateInputData: Prisma.MessageStyleUncheckedUpdateInput = {
        guild_id: request.params.guildId,
        name: body.name,
        colour: body.colour,
        show_author: body.show_author,
        show_image: body.show_image,
        show_thumbnail: body.show_thumbnail,
        show_footer: body.show_footer,
        show_timestamp: body.show_timestamp,
        title_mutator: body.title_mutator || null,
        description_mutator: body.description_mutator || null
    };

    try {
        const updateArgs = { where: { id: Number(body.id) }, data: updateInputData };
        const updateResponse = await prisma.messageStyle.update(updateArgs);
        response.status(200).json(updateResponse);
    } catch (error) {
        logger.error(error);
        const isPrismaError = error instanceof Prisma.PrismaClientKnownRequestError;
        response.status(500).json({ error: isPrismaError ? error.message : error });
    }
};

export const del = async (request: Request, response: Response) => {
    logger.info(`Deleting style(s): ${request.body.ids} - ${request.params.guildId}`)

    const ids = request.body.ids?.map((id: string) => Number(id));

    if (!ids) {
        response.status(400).json({ error: "Couldn't parse ID's from request body" });
        return;
    }

    try {
        const deleteArgs = { where: { guild_id: request.params.guildId, id: { in: ids } } };
        await prisma.messageStyle.deleteMany(deleteArgs);
        response.status(204).send();
    } catch (error) {
        logger.error(error);
        const isPrismaError = error instanceof Prisma.PrismaClientKnownRequestError;
        response.status(500).json({ error: isPrismaError ? error.message : error });
    }
};

export const datatable = async (request: Request, response: Response) => {
    return await datatableRequest(
        request,
        response,
        prisma.messageStyle,
        [{ updated_at: "desc" }, { id: "asc" }],
        { },
        { guild_id: request.params.guildId }  // TODO: verify authenticated user can access this guild
    );
};

export const select = async (request: Request, response: Response) => {
    const data = await prisma.messageStyle.findMany({
        where: {
            guild_id: request.params.guildId,
            name: { contains: `${request.query.search}` }
        }
    });

    // Preline Bug: https://github.com/htmlstreamofficial/preline/issues/567
    // The returned data must have a "title" key, otherwise the advanced
    // select component with 'tags' mode will have no title, regardless of
    // mapping.
    const modifiedResults = data.map(filter => ({
        ...filter,
        title: filter.name
    }));

    response.json(modifiedResults);
};

export default { get, post, patch, del, datatable, select };